@@ -12,7 +12,8 @@
 ;; these methods.
 
 (ns puppetlabs.http.client.async
-  (:import (com.puppetlabs.http.client HttpMethod HttpClientException)
+  (:import (com.puppetlabs.http.client HttpMethod HttpClientException
+                                       RequestOptions)
            (org.apache.http.nio.client HttpAsyncClient)
            (org.apache.http.impl.nio.client HttpAsyncClients)
            (org.apache.http.client.methods HttpGet HttpHead HttpPost HttpPut HttpTrace HttpDelete HttpOptions HttpPatch)
@@ -23,9 +24,11 @@
            (org.apache.http.nio.entity NStringEntity)
            (org.apache.http.entity InputStreamEntity ContentType)
            (java.io InputStream)
-           (com.puppetlabs.http.client.impl Compression)
+           (com.puppetlabs.http.client.impl AsyncClose Compression)
            (org.apache.http.client RedirectStrategy)
-           (org.apache.http.impl.client LaxRedirectStrategy DefaultRedirectStrategy))
+           (org.apache.http.impl.client LaxRedirectStrategy DefaultRedirectStrategy)
+           (org.apache.http.nio.conn.ssl SSLIOSessionStrategy)
+           (org.apache.http.conn.ssl SSLContexts))
   (:require [puppetlabs.certificate-authority.core :as ssl]
             [clojure.string :as str]
             [puppetlabs.kitchensink.core :as ks]
@@ -63,7 +66,7 @@
   [req]
   (initialize-ssl-context-from-ca-pem req))
 
-(schema/defn configure-ssl :- (schema/either {} common/SslContextOptions)
+(schema/defn configure-ssl-ctxt :- (schema/either {} common/SslContextOptions)
   "Configures a request map to have an SSLContext. It will use an existing one
   (stored in :ssl-context) if already present, and will fall back to a set of
   PEM files (stored in :ssl-cert, :ssl-key, and :ssl-ca-cert) if those are present.
@@ -231,8 +234,16 @@
   (try
     (deliver result (callback-response opts callback response))
     (finally
+      ;; Call to AsyncClose/close added for TK-101.  Can't call client close
+      ;; from the current thread context because the Apache HTTP client library
+      ;; may have called through to this function from an i/o reactor thread and
+      ;; the client close might try to do a join on the i/o reactor threads,
+      ;; leading to a deadlock.  AsyncClose does the close on a different thread
+      ;; to avoid the deadlock.  Not a great solution but avoids the deadlock
+      ;; until an implementation that allows the originating request thread to
+      ;; perform the client close can be done.
       (if (not (:persistent opts))
-        (.close client)))))
+        (AsyncClose/close client)))))
 
 (schema/defn future-callback
   [client :- common/Client
@@ -262,6 +273,7 @@
 (schema/defn extract-client-opts :- common/ClientOptions
   [opts :- common/UserRequestOptions]
   (select-keys opts [:ssl-context :ssl-ca-cert :ssl-cert :ssl-key
+                     :ssl-protocols :cipher-suites
                      :force-redirects :follow-redirects]))
 
 (schema/defn extract-ssl-opts :- common/SslOptions
@@ -272,6 +284,18 @@
   [opts :- common/UserRequestOptions]
   (select-keys opts [:url :method :headers :body :decompress-body :as :persistent :query-params]))
 
+(schema/defn ^:always-validate ssl-strategy :- SSLIOSessionStrategy
+  [ssl-ctxt-opts :- common/SslContextOptions
+   ssl-prot-opts :- common/SslProtocolOptions]
+  (SSLIOSessionStrategy.
+    (:ssl-context ssl-ctxt-opts)
+    (if (contains? ssl-prot-opts :ssl-protocols)
+      (into-array String (:ssl-protocols ssl-prot-opts))
+      RequestOptions/DEFAULT_SSL_PROTOCOLS)
+    (if (contains? ssl-prot-opts :cipher-suites)
+      (into-array String (:cipher-suites ssl-prot-opts)))
+    SSLIOSessionStrategy/BROWSER_COMPATIBLE_HOSTNAME_VERIFIER))
+
 (schema/defn ^:always-validate redirect-strategy :- RedirectStrategy
   [opts :- common/ClientOptions]
   (let [follow-redirects (:follow-redirects opts)
@@ -290,13 +314,12 @@
 
 (schema/defn ^:always-validate create-default-client :- common/Client
   [opts :- common/ClientOptions]
-  (let [configured-opts (configure-ssl (extract-ssl-opts opts))
+  (let [ssl-ctxt-opts   (configure-ssl-ctxt (extract-ssl-opts opts))
+        ssl-prot-opts   (select-keys opts [:ssl-protocols :cipher-suites])
         client-builder  (HttpAsyncClients/custom)
-        client          (do (when (:ssl-context configured-opts)
-                              (.setSSLContext client-builder
-                                              (:ssl-context configured-opts)))
-                            (.setRedirectStrategy client-builder
-                                                  (redirect-strategy opts))
+        client          (do (when (:ssl-context ssl-ctxt-opts)
+                              (.setSSLStrategy client-builder (ssl-strategy ssl-ctxt-opts ssl-prot-opts)))
+                            (.setRedirectStrategy client-builder (redirect-strategy opts))
                             (.build client-builder))]
     (.start client)
     client))
@@ -377,15 +400,7 @@
 
 (schema/defn create-client :- common/HTTPClient
   [opts :- common/ClientOptions]
-  (let [configured-opts (configure-ssl (extract-ssl-opts opts))
-        client-builder  (HttpAsyncClients/custom)
-        client          (do (when (:ssl-context configured-opts)
-                              (.setSSLContext client-builder
-                                              (:ssl-context configured-opts)))
-                            (.setRedirectStrategy client-builder
-                                                  (redirect-strategy opts))
-                            (.build client-builder))]
-    (.start client)
+  (let [client (create-default-client opts)]
     (reify common/HTTPClient
       (get [this url] (common/get this url {}))
       (get [_ url opts] (request-with-reified-client (assoc opts :method :get :url url) nil client))
