@@ -1,8 +1,8 @@
-(defproject puppetlabs.packages/puppetserver "0.2.1"
+(defproject puppetlabs.packages/puppetserver "0.4.0"
   :description "Release artifacts for puppet-server"
   :pedantic? :abort
-  :dependencies [[puppetlabs/puppet-server "0.2.1"]
-                 [puppetlabs/trapperkeeper-webserver-jetty9 "0.7.5"]]
+  :dependencies [[puppetlabs/puppet-server "0.4.0"]
+                 [puppetlabs/trapperkeeper-webserver-jetty9 "0.9.0"]]
 
   :uberjar-name "puppet-server-release.jar"
 
@@ -13,6 +13,6 @@
   :ezbake { :user "puppet"
             :group "puppet"
             :build-type "foss"
-            :java-args "-Xms2g -Xmx2g"
+            :java-args "-Xms2g -Xmx2g -XX:MaxPermSize=256m"
             }
   )
