@@ -5,16 +5,21 @@ java_import org.slf4j.LoggerFactory
 
 Puppet::Util::Log.newdesttype :logback do
   def handle(msg)
+    output = msg.message
+    if msg.source.size > 0
+      output = "#{msg.source} #{output}"
+    end
+
     logger = LoggerFactory.getLogger("puppet-server")
     case msg.level
       when :debug
-        logger.debug(msg.message)
+        logger.debug(output)
       when :info, :notice
-        logger.info(msg.message)
+        logger.info(output)
       when :warning
-        logger.warn(msg.message)
+        logger.warn(output)
       when :err, :alert, :emerg, :crit
-        logger.error(msg.message)
+        logger.error(output)
     end
   end
 end
