@@ -1,6 +1,7 @@
 (ns puppetlabs.services.master.master-core
+  (:import (java.io FileInputStream))
   (:require [compojure.core :as compojure]
-            [compojure.route :as route]
+            [me.raynes.fs :as fs]
             [puppetlabs.puppetserver.ringutils :as ringutils]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -47,6 +48,12 @@
     (compojure/GET "/resource_type/*" request
                    (request-handler request))
     (compojure/GET "/resource_types/*" request
+                   (request-handler request))
+
+    ;; TODO: when we get rid of the legacy dashboard after 3.4, we should remove
+    ;; this endpoint as well.  It makes more sense for this type of query to be
+    ;; directed to PuppetDB.
+    (compojure/GET "/facts_search/*" request
                    (request-handler request))))
 
 (defn root-routes
@@ -58,6 +65,32 @@
     (compojure/context "/:environment" [environment]
                        (legacy-routes request-handler))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Lifecycle Helper Functions
+
+(defn validate-memory-requirements!
+  "On Linux Distributions, parses the /proc/meminfo file to determine
+   the total amount of System RAM, and throws an exception if that
+   is less than 1.1 times the maximum heap size of the JVM. This is done
+   so that the JVM doesn't fail later due to an Out of Memory error."
+  []
+  (when (fs/exists? "/proc/meminfo")
+    ; Due to OpenJDK Issue JDK-7132461
+    ; (https://bugs.openjdk.java.net/browse/JDK-7132461),
+    ; we have to open and slurp a FileInputStream object rather than
+    ; slurping the file directly, since directly slurping the file
+    ; causes a call to be made to FileInputStream.available().
+    (with-open [mem-info-file (FileInputStream. "/proc/meminfo")]
+      (let [heap-size (/ (.maxMemory (Runtime/getRuntime)) 1024)
+            mem-size (Integer. (second (re-find #"MemTotal:\s+(\d+)\s+\S+"
+                                                (slurp mem-info-file))))
+            required-mem-size (/ heap-size 0.9)]
+        (when (< mem-size required-mem-size)
+          (throw (Error.
+                   (str "Not enough RAM. Puppet Server requires at least "
+                        (int (/ required-mem-size 1024.0))
+                        "MB of RAM."))))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
